@@ -1,6 +1,6 @@
-/*	$Id: ksql.h,v 1.13 2017/06/27 13:58:04 kristaps Exp $ */
+/*	$Id: ksql.h,v 1.26 2018/03/09 06:40:54 kristaps Exp $ */
 /*
- * Copyright (c) 2016--2017 Kristaps Dzonsons <kristaps@bsd.lv>
+ * Copyright (c) 2016--2018 Kristaps Dzonsons <kristaps@bsd.lv>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -17,10 +17,50 @@
 #ifndef KSQL_H
 #define KSQL_H
 
+/*
+ * Stringification of version major, minor, and build.
+ * I have on idea if this is necessary.
+ */
+#define NAME(s) NAME0(s)
+#define NAME0(s) #s
+#define NAME2(x,y,z) x ## . ## y ## . ## z
+#define NAME1(x,y,z) NAME2(x,y,z)
+
+/*
+ * Major version.
+ */
+#define	KSQL_VMAJOR	0
+
+/*
+ * Minor version.
+ */
+#define	KSQL_VMINOR	2
+
+/*
+ * Build version.
+ */
+#define	KSQL_VBUILD	5
+
+/*
+ * Version string of major.minor.build (as a literal string).
+ */
+#define	KSQL_VERSION	NAME(NAME1(KSQL_VMAJOR,KSQL_VMINOR,KSQL_VBUILD))
+
+/*
+ * Integral stamp of version.
+ * Guaranteed to be increasing with build, minor, and major.
+ * (Assumes build and minor never go over 100.)
+ */
+#define	KSQL_VSTAMP \
+	((KSQL_VBUILD+1) + \
+	 (KSQL_VMINOR+1)*100 + \
+	 (KSQL_VMAJOR+1)*10000)
+
 /*
  * Error codes returned by all functions.
  * In general, checking for zero means success.
  */
+
 enum ksqlc {
 	KSQL_OK = 0, /* success */
 	KSQL_DONE, /* data done */
@@ -39,7 +79,25 @@ enum ksqlc {
 typedef	void (*ksqldbmsg)(void *, int, int, const char *, const char *);
 typedef	void (*ksqlmsg)(void *, enum ksqlc, const char *, const char *);
 
+struct	ksqlrole {
+	const int	  *roles; /* roles we can access */
+	const int	  *stmts; /* statements we can call */
+};
+
+struct	ksqlroles {
+	struct ksqlrole	  *roles;
+	size_t		   rolesz;
+	size_t		   defrole;
+};
+
+struct	ksqlstmts {
+	const char *const *stmts;
+	size_t		   stmtsz;
+};
+
 struct	ksqlcfg {
+	struct ksqlstmts  stmts;
+	struct ksqlroles  roles;
 	unsigned int	  flags;
 #define	KSQL_EXIT_ON_ERR  0x01
 #define	KSQL_FOREIGN_KEYS 0x02
@@ -63,11 +121,13 @@ enum ksqlc	 ksql_bind_int(struct ksqlstmt *, size_t, int64_t);
 enum ksqlc	 ksql_bind_null(struct ksqlstmt *, size_t);
 enum ksqlc	 ksql_bind_str(struct ksqlstmt *, size_t, const char *);
 enum ksqlc	 ksql_bind_zblob(struct ksqlstmt *, size_t, size_t);
+void		 ksql_cfg_defaults(struct ksqlcfg *);
 enum ksqlc	 ksql_close(struct ksql *);
 enum ksqlc	 ksql_exec(struct ksql *, const char *, size_t);
 enum ksqlc	 ksql_free(struct ksql *);
 enum ksqlc	 ksql_lastid(struct ksql *, int64_t *);
 enum ksqlc	 ksql_open(struct ksql *, const char *);
+void		 ksql_role(struct ksql *, size_t);
 enum ksqlc	 ksql_stmt_alloc(struct ksql *, 
 			struct ksqlstmt **, const char *, size_t);
 const void	*ksql_stmt_blob(struct ksqlstmt *, size_t);
@@ -84,6 +144,7 @@ enum ksqlc	 ksql_trans_commit(struct ksql *, size_t);
 enum ksqlc	 ksql_trans_exclopen(struct ksql *, size_t);
 enum ksqlc	 ksql_trans_open(struct ksql *, size_t);
 enum ksqlc	 ksql_trans_rollback(struct ksql *, size_t);
+enum ksqlc	 ksql_trans_singleopen(struct ksql *, size_t);
 
 void		 ksql_trace(struct ksql *);
 void		 ksql_untrace(struct ksql *);
